@@ -5,10 +5,6 @@
 #include <QTime>
 #include <QSettings>
 
-#ifdef Q_WS_MAEMO_5
-#include <QtMaemo5>
-#endif
-
 #include "connectdialog.h"
 
 //
@@ -36,12 +32,20 @@
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 MainWindow::MainWindow(QWidget *parent)
-    : QMainWindow(parent), ui(new Ui::MainWindow), qmpd(0),
+    : QMainWindow(parent), ui(new Ui::MainWindow), qmpd_(0),
     playlistModel_(new SongModel), currentSong_(0), scrobbled_(false),
     plDelegate_(new PlaylistItemDelegate(this)),
     codec_(QTextCodec::codecForLocale()),
     fileModel_(new SongSortFilterModel(this))
 {
+#ifdef Q_WS_MAEMO_5
+    // DBus connection for orientation change
+    QDBusConnection::systemBus().connect(QString(), MCE_SIGNAL_PATH, MCE_SIGNAL_IF,
+                                         MCE_DEVICE_ORIENTATION_SIG,
+                                         this,
+                                         SLOT(orientationChanged(QString)));
+#endif
+
     ui->setupUi(this);
 
     // Some signals and slots together
@@ -120,8 +124,8 @@ MainWindow::MainWindow(QWidget *parent)
 
 MainWindow::~MainWindow()
 {
-    if (qmpd) {
-        delete qmpd;
+    if (qmpd_) {
+        delete qmpd_;
     }
     if ( currentSong_ ) {
         delete currentSong_;
@@ -184,6 +188,44 @@ void MainWindow::saveAppState() const {
     sett.setValue("app/hidden", this->isHidden());
 }
 
+//
+// Orientation changing stuff for maemo 5
+//
+#ifdef Q_WS_MAEMO_5
+void MainWindow::setLandscape() {
+    setAttribute(Qt::WA_Maemo5ForceLandscapeOrientation, true);
+}
+void MainWindow::setPortrait() {
+    setAttribute(Qt::WA_Maemo5ForcePortraitOrientation, true);
+}
+void MainWindow::orientationChanged(const QString &newOrientation)
+{
+    if (newOrientation == QLatin1String(MCE_ORIENTATION_PORTRAIT))
+        setPortrait();
+    else
+        setLandscape();
+}
+bool MainWindow::event(QEvent *ev) {
+    switch (ev->type()) {
+    case QEvent::WindowActivate:
+        QDBusConnection::systemBus().call(
+                QDBusMessage::createMethodCall(MCE_SERVICE, MCE_REQUEST_PATH,
+                                               MCE_REQUEST_IF,
+                                               MCE_ACCELEROMETER_ENABLE_REQ));
+        break;
+    case QEvent::WindowDeactivate:
+        QDBusConnection::systemBus().call(
+                QDBusMessage::createMethodCall(MCE_SERVICE, MCE_REQUEST_PATH,
+                                               MCE_REQUEST_IF,
+                                               MCE_ACCELEROMETER_DISABLE_REQ));
+        break;
+    default:
+        break;
+    }
+    return QWidget::event(ev);
+}
+#endif
+
 //
 // Restore size and position of the app
 //
@@ -305,36 +347,36 @@ bool MainWindow::setQmpd(Qmpd *qm) {
     if (!qm) {
         return false;
     }
-    qmpd = qm;
+    qmpd_ = qm;
 
     // Let's connect some signals
-    connect(qmpd, SIGNAL(mpdStatusChanged(MpdObj*,ChangedStatusType)), this, SLOT(onMpdStatusChange(MpdObj*,ChangedStatusType)));
-    connect(qmpd, SIGNAL(mpdErrorOccured(MpdObj*)), this, SLOT(onMpdError(MpdObj*)));
-    connect(ui->action_Update_database, SIGNAL(triggered()), qmpd, SLOT(onUpdateDB()));
+    connect(qmpd_, SIGNAL(mpdStatusChanged(MpdObj*,ChangedStatusType)), this, SLOT(onMpdStatusChange(MpdObj*,ChangedStatusType)));
+    connect(qmpd_, SIGNAL(mpdErrorOccured(MpdObj*)), this, SLOT(onMpdError(MpdObj*)));
+    connect(ui->action_Update_database, SIGNAL(triggered()), qmpd_, SLOT(onUpdateDB()));
 
     // Play/pause etc... controls
-    connect(ui->playpauseButton, SIGNAL(clicked()), qmpd, SLOT(onPlaypauseClick()));
-    connect(ui->stopButton, SIGNAL(clicked()), qmpd, SLOT(onStopClick()));
-    connect(ui->prevButton, SIGNAL(clicked()), qmpd, SLOT(onPrevClick()));
-    connect(ui->nextButton, SIGNAL(clicked()), qmpd, SLOT(onNextClick()));
-    connect(ui->songProgressSlider, SIGNAL(sliderMoved(int)), qmpd, SLOT(onSliderMoved(int)));
-    connect(ui->songProgressSlider, SIGNAL(sliderPressed()), qmpd, SLOT(onSliderPress()));
-    connect(ui->songProgressSlider, SIGNAL(sliderReleased()), qmpd, SLOT(onSliderRelease()));
+    connect(ui->playpauseButton, SIGNAL(clicked()), qmpd_, SLOT(onPlaypauseClick()));
+    connect(ui->stopButton, SIGNAL(clicked()), qmpd_, SLOT(onStopClick()));
+    connect(ui->prevButton, SIGNAL(clicked()), qmpd_, SLOT(onPrevClick()));
+    connect(ui->nextButton, SIGNAL(clicked()), qmpd_, SLOT(onNextClick()));
+    connect(ui->songProgressSlider, SIGNAL(sliderMoved(int)), qmpd_, SLOT(onSliderMoved(int)));
+    connect(ui->songProgressSlider, SIGNAL(sliderPressed()), qmpd_, SLOT(onSliderPress()));
+    connect(ui->songProgressSlider, SIGNAL(sliderReleased()), qmpd_, SLOT(onSliderRelease()));
 
     // Playlist
-    connect(ui->playlistDelAllButton, SIGNAL(clicked()), qmpd, SLOT(onClearPlaylist()));
+    connect(ui->playlistDelAllButton, SIGNAL(clicked()), qmpd_, SLOT(onClearPlaylist()));
     connect(ui->playlistDelSelButton, SIGNAL(clicked()), this, SLOT(onDeleteSelectedFromPlaylist()));
-    connect(qmpd, SIGNAL(updatePlaylistNow(MpdObj*)), this, SLOT(updatePlaylist(MpdObj*)));
+    connect(qmpd_, SIGNAL(updatePlaylistNow(MpdObj*)), this, SLOT(updatePlaylist(MpdObj*)));
     connect(ui->playlistTableview, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(onPlaylistDoubleClick(QModelIndex)));
 
     ui->playpauseButton->setShortcut(Qt::SHIFT + Qt::Key_P);
 
     // Tray icon context menu actions
 #if !defined (Q_WS_MAEMO_5) && !defined (Q_WS_HILDON)
-    connect(trayPlayAction_, SIGNAL(triggered()), qmpd, SLOT(onPlaypauseClick()));
-    connect(trayStopAction_, SIGNAL(triggered()), qmpd, SLOT(onStopClick()));
-    connect(trayNextAction_, SIGNAL(triggered()), qmpd, SLOT(onNextClick()));
-    connect(trayPrevAction_, SIGNAL(triggered()), qmpd, SLOT(onPrevClick()));
+    connect(trayPlayAction_, SIGNAL(triggered()), qmpd_, SLOT(onPlaypauseClick()));
+    connect(trayStopAction_, SIGNAL(triggered()), qmpd_, SLOT(onStopClick()));
+    connect(trayNextAction_, SIGNAL(triggered()), qmpd_, SLOT(onNextClick()));
+    connect(trayPrevAction_, SIGNAL(triggered()), qmpd_, SLOT(onPrevClick()));
 
 #endif
 
@@ -775,9 +817,9 @@ void MainWindow::onDisconnect() {
     if ( collectionBuilder_.isRunning() )
         collectionBuilder_.exit();
 
-    if ( qmpd ) {
-        delete qmpd;
-        qmpd = 0;
+    if ( qmpd_ ) {
+        delete qmpd_;
+        qmpd_ = 0;
     }
 
     // Clear playlist
@@ -800,7 +842,11 @@ void MainWindow::onDisconnect() {
     // Clear file browser
     FBDirTree_.clear();
     qDebug() << "FBDirTree count = " << FBDirTree_.count();
-    FBModel_->setDirTree(FBDirTree_);
+    qDebug() << "Crashed yet 1?";
+    if ( FBModel_) {
+        FBModel_->setDirTree(FBDirTree_);
+    }
+    qDebug() << "Crashed yet 2?";
     //ui->directoryTree->setExpanded(ui->directoryTree->rootIndex(), false);
 
     // Set texts
@@ -834,7 +880,7 @@ void MainWindow::onDisconnect() {
 // Menu item "connect->localhost"
 //
 void MainWindow::onConnectLocalhost() {
-    if ( qmpd ) {
+    if ( qmpd_ ) {
         onDisconnect();
     }
     connectMPD("localhost", 6600);
@@ -847,7 +893,7 @@ void MainWindow::onConnectOther() {
     // Dialog
     ConnectDialog dlg(this);
     if ( dlg.exec() ) {
-        if ( qmpd ) {
+        if ( qmpd_ ) {
             onDisconnect();
         }
 
@@ -872,10 +918,10 @@ void MainWindow::onConnectOther() {
 // Tries to connect to MPD
 //
 bool MainWindow::connectMPD(QString host, int port, QString passwd) {
-    qmpd = new Qmpd(host, port, passwd);
-    if (qmpd->isConnected()) {
-        if (!setQmpd(qmpd)) {
-            //qDebug() << "FATAL ERROR: Couldn't set Qmpd for main window!";
+    qmpd_ = new Qmpd(host, port, passwd);
+    if (qmpd_->isConnected()) {
+        if (!setQmpd(qmpd_)) {
+            qDebug() << "FATAL ERROR: Couldn't set Qmpd for main window!";
             myShowMessage(tr("Could not connect to MPD, check your settings."));
             onDisconnect();
             return false;
@@ -891,7 +937,7 @@ bool MainWindow::connectMPD(QString host, int port, QString passwd) {
     myShowMessage(tr("Connected to MPD succesfully!"));
 
     // Get playlist first
-    updatePlaylist(qmpd->getMpdObj());
+    updatePlaylist(qmpd_->getMpdObj());
 
     // filebrowser
     initializeFileBrowser();
@@ -1071,8 +1117,8 @@ void MainWindow::onDeleteSelectedFromPlaylist() {
             positions.append(pos);
         }
 
-        if ( qmpd )
-            qmpd->removeFromPlaylist(positions);
+        if ( qmpd_ )
+            qmpd_->removeFromPlaylist(positions);
 
         if ( indexes.count() > 0 ) {
             myShowMessage(tr("Removed ") + QString::number(indexes.count()) + tr(" songs from playlist"));
@@ -1088,8 +1134,8 @@ void MainWindow::onDeleteSelectedFromPlaylist() {
 //
 void MainWindow::onPlaylistDoubleClick(const QModelIndex & index) {
     int pos = index.row();
-    if ( qmpd )
-        qmpd->playFromPos(pos);
+    if ( qmpd_ )
+        qmpd_->playFromPos(pos);
 }
 
 //
@@ -1114,8 +1160,8 @@ void MainWindow::onCutSelected() {
         }
 
         // remove from playlist
-        if ( qmpd )
-            qmpd->removeFromPlaylist(positions);
+        if ( qmpd_ )
+            qmpd_->removeFromPlaylist(positions);
 
         if ( indexes.count() > 0 ) {
             myShowMessage(tr("Cut ") + QString::number(indexes.count()) + tr(" songs to clipboard"));
@@ -1146,14 +1192,14 @@ void MainWindow::onPasteSongs() {
 
         // Add songs to playlist
         foreach (QString tmppath, myClipboard_) {
-            mpd_playlist_queue_add(qmpd->getMpdObj(), tmppath.toLocal8Bit());
+            mpd_playlist_queue_add(qmpd_->getMpdObj(), tmppath.toLocal8Bit());
         }
-        mpd_playlist_queue_commit(qmpd->getMpdObj());
+        mpd_playlist_queue_commit(qmpd_->getMpdObj());
 
         // Move added songs to right position
-        int playlistLen = mpd_playlist_get_playlist_length(qmpd->getMpdObj());
+        int playlistLen = mpd_playlist_get_playlist_length(qmpd_->getMpdObj());
         for (int i = 0 ; i <  myClipboard_.count() ; ++i) {
-            mpd_playlist_move_pos(qmpd->getMpdObj(), playlistLen-1, pos+1);
+            mpd_playlist_move_pos(qmpd_->getMpdObj(), playlistLen-1, pos+1);
         }
 
         // Message
@@ -1223,9 +1269,9 @@ void MainWindow::onAddArtist() {
 
         // Add to playlist
         foreach ( QmpdSong song, songs ) {
-            mpd_playlist_queue_add(qmpd->getMpdObj(), song.file_.toLocal8Bit());
+            mpd_playlist_queue_add(qmpd_->getMpdObj(), song.file_.toLocal8Bit());
         }
-        mpd_playlist_queue_commit(qmpd->getMpdObj());
+        mpd_playlist_queue_commit(qmpd_->getMpdObj());
         myShowMessage(tr("Added all songs from artist \"") + artist + tr("\" to playlist!"));
     }
 }
@@ -1272,9 +1318,9 @@ void MainWindow::onAddAlbum() {
 
         // Add to playlist
         foreach ( QmpdSong song, songs ) {
-            mpd_playlist_queue_add(qmpd->getMpdObj(), song.file_.toLocal8Bit());
+            mpd_playlist_queue_add(qmpd_->getMpdObj(), song.file_.toLocal8Bit());
         }
-        mpd_playlist_queue_commit(qmpd->getMpdObj());
+        mpd_playlist_queue_commit(qmpd_->getMpdObj());
         myShowMessage(tr("Added songs from ") + QString::number(albums.count()) + tr(" albums to playlist!"));
     }
 }
@@ -1326,9 +1372,9 @@ void MainWindow::onAddTrack() {
 
         // Add selected indexes to playlist
         foreach ( index, indexes ) {
-            mpd_playlist_queue_add(qmpd->getMpdObj(), songs.at(index.row()).file_.toLocal8Bit());
+            mpd_playlist_queue_add(qmpd_->getMpdObj(), songs.at(index.row()).file_.toLocal8Bit());
         }
-        mpd_playlist_queue_commit(qmpd->getMpdObj());
+        mpd_playlist_queue_commit(qmpd_->getMpdObj());
         myShowMessage(tr("Added ") + QString::number(indexes.count()) + tr(" songs from \"") + artist + " - " + album + tr("\" to playlist!"));
     }
 }
@@ -1344,7 +1390,7 @@ void MainWindow::buildCollectionNow() {
     ui->tabCollection->setEnabled(false);
     ui->filebrowserTab->setEnabled(false);
 
-    collectionBuilder_.setMpd(qmpd->getMpdObj());
+    collectionBuilder_.setMpd(qmpd_->getMpdObj());
 
     // Check if mpd database updated after our last collection build
     // if ( mpd_updated > collection_build ) collectionBuilder_.setBuildFromFile(false);
@@ -1478,7 +1524,7 @@ bool MainWindow::buildFBTreePath(QString path) {
         return false;
     }
 
-    MpdData* dirdata = mpd_database_get_directory(qmpd->getMpdObj(), path.toLocal8Bit());
+    MpdData* dirdata = mpd_database_get_directory(qmpd_->getMpdObj(), path.toLocal8Bit());
     QStringList spaths;
     if ( dirdata ) {
         do {
@@ -1506,7 +1552,7 @@ void MainWindow::onFBDirClicked(QModelIndex index) {
     // Show files
     QStringList subdirs;
     SongModel* newmodel = new SongModel();
-    MpdData* filedata = mpd_database_get_directory(qmpd->getMpdObj(), fullpath.toLocal8Bit());
+    MpdData* filedata = mpd_database_get_directory(qmpd_->getMpdObj(), fullpath.toLocal8Bit());
     if ( filedata ) {
         do {
             if ( filedata->type == MPD_DATA_TYPE_SONG ) {
@@ -1589,7 +1635,7 @@ void MainWindow::onUpdateDir() {
         QModelIndex index = indexes.at(0);
         QString fullpath = index.data(Qt::UserRole).toString();
         //qDebug() << "Update dir: " << fullpath;
-        mpd_database_update_dir(qmpd->getMpdObj(), fullpath.toLocal8Bit());
+        mpd_database_update_dir(qmpd_->getMpdObj(), fullpath.toLocal8Bit());
     }
 }
 
@@ -1602,8 +1648,8 @@ void MainWindow::onAddDir() {
         QModelIndexList indexes = sm->selectedRows();
         QModelIndex index = indexes.at(0);
         QString fullpath = index.data(Qt::UserRole).toString();
-        mpd_playlist_queue_add(qmpd->getMpdObj(), fullpath.toLocal8Bit());
-        mpd_playlist_queue_commit(qmpd->getMpdObj());
+        mpd_playlist_queue_add(qmpd_->getMpdObj(), fullpath.toLocal8Bit());
+        mpd_playlist_queue_commit(qmpd_->getMpdObj());
         myShowMessage(tr("Added all songs from \"") + fullpath + tr("\" and all its subdirectories!"));
     }
 }
@@ -1617,9 +1663,9 @@ void MainWindow::onAddTrackFB() {
         QModelIndexList indexes = sm->selectedRows();
         foreach (QModelIndex index, indexes) {
             QString fullpath = index.data(Qt::UserRole).toString();
-            mpd_playlist_queue_add(qmpd->getMpdObj(), fullpath.toLocal8Bit());
+            mpd_playlist_queue_add(qmpd_->getMpdObj(), fullpath.toLocal8Bit());
         }
-        mpd_playlist_queue_commit(qmpd->getMpdObj());
+        mpd_playlist_queue_commit(qmpd_->getMpdObj());
         myShowMessage(tr("Added ") + QString::number(indexes.count()) + tr(" songs to playlist"));
     }
 }
